package sun.net.www.protocol.https;

import java.net.InetSocketAddress;
import javax.net.ssl.HandshakeCompletedEvent;
import java.security.Principal;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import javax.net.ssl.SSLPeerUnverifiedException;
import java.security.cert.X509Certificate;
import sun.security.util.HostnameChecker;
import java.net.UnknownHostException;
import javax.net.ssl.SSLParameters;
import java.io.UnsupportedEncodingException;
import java.io.PrintStream;
import java.io.BufferedOutputStream;
import sun.security.ssl.SSLSocketImpl;
import java.net.SocketException;
import java.net.Socket;
import java.net.InetAddress;
import sun.util.logging.PlatformLogger;
import sun.net.www.protocol.http.HttpURLConnection;
import java.net.Proxy;
import java.io.IOException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.StringTokenizer;
import java.util.Vector;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HandshakeCompletedListener;
import sun.net.www.http.HttpClient;

final class HttpsClient extends HttpClient implements HandshakeCompletedListener
{
    private static final int httpsPortNumber = 443;
    private static final String defaultHVCanonicalName = "javax.net.ssl.HttpsURLConnection.DefaultHostnameVerifier";
    private static final String TLS13_CIPHER1 = "TLS_AES_128_GCM_SHA256";
    private static final String TLS13_CIPHER2 = "TLS_AES_256_GCM_SHA384";
    private HostnameVerifier hv;
    private SSLSocketFactory sslSocketFactory;
    private SSLSession session;
    
    @Override
    protected int getDefaultPort() {
        return 443;
    }
    
    private String[] getCipherSuites(String[] a, SSLSocket sslSocket) {
        String s = (String)AccessController.doPrivileged(new GetPropertyAction("https.cipherSuites"));
        if (s == null || "".equals(s)) {
            if (a == null || !Arrays.asList(a).contains("TLSv1.3")) {
                return null;
            }
            List<String> list = Arrays.asList(sslSocket.getEnabledCipherSuites());
            if (!list.contains("TLS_AES_128_GCM_SHA256") && !list.contains("TLS_AES_256_GCM_SHA384")) {
                List<String> list2 = Arrays.asList(sslSocket.getSupportedCipherSuites());
                ArrayList<Object> list3 = new ArrayList();
                if (list2.contains("TLS_AES_128_GCM_SHA256")) {
                    list3.add("TLS_AES_128_GCM_SHA256");
                }
                if (list2.contains("TLS_AES_256_GCM_SHA384")) {
                    list3.add("TLS_AES_256_GCM_SHA384");
                }
                Iterator<Object> iterator = list.iterator();
                while (iterator.hasNext()) {
                    list3.add((Object)iterator.next());
                }
                return (String[])list3.toArray(new String[0]);
            }
            else {
                return null;
            }
        }
        else {
            Vector<String> vector = new Vector();
            StringTokenizer stringTokenizer = new StringTokenizer(s, ",");
            while (stringTokenizer.hasMoreTokens()) {
                vector.addElement(stringTokenizer.nextToken());
            }
            String[] array = new String[vector.size()];
            for (int i = 0; i < array.length; ++i) {
                array[i] = (String)vector.elementAt(i);
            }
            return array;
        }
    }
    
    private String[] getProtocols() {
        String s = (String)AccessController.doPrivileged(new GetPropertyAction("https.protocols"));
        if (s == null || "".equals(s)) {
            return null;
        }
        Vector<String> vector = new Vector();
        StringTokenizer stringTokenizer = new StringTokenizer(s, ",");
        while (stringTokenizer.hasMoreTokens()) {
            vector.addElement(stringTokenizer.nextToken());
        }
        String[] array = new String[vector.size()];
        for (int i = 0; i < array.length; ++i) {
            array[i] = (String)vector.elementAt(i);
        }
        return array;
    }
    
    private String getUserAgent() {
        String s = (String)AccessController.doPrivileged(new GetPropertyAction("https.agent"));
        if (s == null || s.length() == 0) {
            s = "JSSE";
            return s;
        }
        return s;
    }
    
    private HttpsClient(SSLSocketFactory sslSocketFactory, URL url) throws IOException {
        this(sslSocketFactory, url, (String)null, -1);
    }
    
    HttpsClient(SSLSocketFactory sslSocketFactory, URL url, String s, int n) throws IOException {
        this(sslSocketFactory, url, s, n, -1);
    }
    
    HttpsClient(SSLSocketFactory sslSocketFactory, URL url, String proxyHost, int proxyPort, int n) throws IOException {
        this(sslSocketFactory, url, (proxyHost == null) ? null : HttpClient.newHttpProxy(proxyHost, proxyPort, "https"), n);
    }
    
    HttpsClient(SSLSocketFactory sslSocketFactory, URL url, Proxy proxy, int n) throws IOException {
        PlatformLogger httpLogger = HttpURLConnection.getHttpLogger();
        if (httpLogger.isLoggable(PlatformLogger.Level.FINEST)) {
            httpLogger.finest("Creating new HttpsClient with url:" + url + " and proxy:" + proxy + " with connect timeout:" + n);
        }
        this.proxy = proxy;
        this.setSSLSocketFactory(sslSocketFactory);
        this.proxyDisabled = true;
        this.host = url.getHost();
        this.url = url;
        this.port = url.getPort();
        if (this.port == -1) {
            this.port = this.getDefaultPort();
        }
        this.setConnectTimeout(n);
        this.openServer();
    }
    
    static HttpClient New(SSLSocketFactory sslSocketFactory, URL url, HostnameVerifier hostnameVerifier, HttpURLConnection httpURLConnection) throws IOException {
        return New(sslSocketFactory, url, hostnameVerifier, true, httpURLConnection);
    }
    
    static HttpClient New(SSLSocketFactory sslSocketFactory, URL url, HostnameVerifier hostnameVerifier, boolean b, HttpURLConnection httpURLConnection) throws IOException {
        return New(sslSocketFactory, url, hostnameVerifier, (String)null, -1, b, httpURLConnection);
    }
    
    static HttpClient New(SSLSocketFactory sslSocketFactory, URL url, HostnameVerifier hostnameVerifier, String s, int n, HttpURLConnection httpURLConnection) throws IOException {
        return New(sslSocketFactory, url, hostnameVerifier, s, n, true, httpURLConnection);
    }
    
    static HttpClient New(SSLSocketFactory sslSocketFactory, URL url, HostnameVerifier hostnameVerifier, String s, int n, boolean b, HttpURLConnection httpURLConnection) throws IOException {
        return New(sslSocketFactory, url, hostnameVerifier, s, n, b, -1, httpURLConnection);
    }
    
    static HttpClient New(SSLSocketFactory sslSocketFactory, URL url, HostnameVerifier hostnameVerifier, String proxyHost, int proxyPort, boolean b, int n, HttpURLConnection httpURLConnection) throws IOException {
        return New(sslSocketFactory, url, hostnameVerifier, (proxyHost == null) ? null : HttpClient.newHttpProxy(proxyHost, proxyPort, "https"), b, n, httpURLConnection);
    }
    
    static HttpClient New(SSLSocketFactory obj, URL url, HostnameVerifier hostnameVerifier, Proxy no_PROXY, boolean b, int n, HttpURLConnection httpURLConnection) throws IOException {
        if (no_PROXY == null) {
            no_PROXY = Proxy.NO_PROXY;
        }
        PlatformLogger httpLogger = HttpURLConnection.getHttpLogger();
        if (httpLogger.isLoggable(PlatformLogger.Level.FINEST)) {
            httpLogger.finest("Looking for HttpClient for URL " + url + " and proxy value of " + no_PROXY);
        }
        Object o = null;
        if (b) {
            o = (HttpClient)HttpsClient.kac.get(url, obj);
            if (o != null && httpURLConnection != null && httpURLConnection.streaming() && httpURLConnection.getRequestMethod() == "POST" && !o.available()) {
                o = null;
            }
            if (o != null) {
                if ((o.proxy != null && o.proxy.equals(no_PROXY)) || (o.proxy == null && no_PROXY == Proxy.NO_PROXY)) {
                    synchronized (o) {
                        o.cachedHttpClient = true;
                        assert !(!o.inCache);
                        o.inCache = false;
                        if (httpURLConnection != null && o.needsTunneling()) {
                            httpURLConnection.setTunnelState(HttpURLConnection.TunnelState.TUNNELING);
                        }
                        if (httpLogger.isLoggable(PlatformLogger.Level.FINEST)) {
                            httpLogger.finest("KeepAlive stream retrieved from the cache, " + o);
                        }
                    }
                }
                else {
                    synchronized (o) {
                        if (httpLogger.isLoggable(PlatformLogger.Level.FINEST)) {
                            httpLogger.finest("Not returning this connection to cache: " + o);
                        }
                        o.inCache = false;
                        o.closeServer();
                    }
                    o = null;
                }
            }
        }
        if (o == null) {
            o = new HttpsClient(obj, url, no_PROXY, n);
        }
        else {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                if (o.proxy == Proxy.NO_PROXY || o.proxy == null) {
                    securityManager.checkConnect(InetAddress.getByName(url.getHost()).getHostAddress(), url.getPort());
                }
                else {
                    securityManager.checkConnect(url.getHost(), url.getPort());
                }
            }
            o.url = url;
        }
        o.setHostnameVerifier(hostnameVerifier);
        return o;
    }
    
    void setHostnameVerifier(HostnameVerifier hv) {
        this.hv = hv;
    }
    
    void setSSLSocketFactory(SSLSocketFactory sslSocketFactory) {
        this.sslSocketFactory = sslSocketFactory;
    }
    
    SSLSocketFactory getSSLSocketFactory() {
        return this.sslSocketFactory;
    }
    
    @Override
    protected Socket createSocket() throws IOException {
        try {
            return this.sslSocketFactory.createSocket();
        }
        catch (SocketException ex) {
            Throwable cause = ex.getCause();
            if (cause != null && cause instanceof UnsupportedOperationException) {
                return super.createSocket();
            }
            throw ex;
        }
    }
    
    @Override
    public boolean needsTunneling() {
        return this.proxy != null && this.proxy.type() != Proxy.Type.DIRECT && this.proxy.type() != Proxy.Type.SOCKS;
    }
    
    @Override
    public void afterConnect() throws IOException, UnknownHostException {
        if (!this.isCachedConnection()) {
            SSLSocketFactory sslSocketFactory = this.sslSocketFactory;
            SSLSocket serverSocket;
            try {
                if (!(this.serverSocket instanceof SSLSocket)) {
                    serverSocket = (SSLSocket)sslSocketFactory.createSocket(this.serverSocket, this.host, this.port, true);
                }
                else {
                    serverSocket = (SSLSocket)this.serverSocket;
                    if ((serverSocket instanceof SSLSocketImpl)) {
                        ((SSLSocketImpl)serverSocket).setHost(this.host);
                    }
                }
            }
            catch (IOException ex) {
                try {
                    serverSocket = (SSLSocket)sslSocketFactory.createSocket(this.host, this.port);
                }
                catch (IOException ex2) {
                    throw ex;
                }
            }
            String[] protocols = this.getProtocols();
            String[] cipherSuites = this.getCipherSuites(protocols, serverSocket);
            if (protocols != null) {
                serverSocket.setEnabledProtocols(protocols);
            }
            if (cipherSuites != null) {
                serverSocket.setEnabledCipherSuites(cipherSuites);
            }
            serverSocket.addHandshakeCompletedListener(this);
            boolean n = true;
            String endpointIdentificationAlgorithm = serverSocket.getSSLParameters().getEndpointIdentificationAlgorithm();
            if (endpointIdentificationAlgorithm != null && endpointIdentificationAlgorithm.length() != 0) {
                if (endpointIdentificationAlgorithm.equalsIgnoreCase("HTTPS")) {
                    n = false;
                }
            }
            else {
                boolean n2 = false;
                if (this.hv != null) {
                    String canonicalName = this.hv.getClass().getCanonicalName();
                    if (canonicalName != null && canonicalName.equalsIgnoreCase("javax.net.ssl.HttpsURLConnection.DefaultHostnameVerifier")) {
                        n2 = true;
                    }
                }
                else {
                    n2 = true;
                }
                if (!(n2 == 0)) {
                    SSLParameters sslParameters = serverSocket.getSSLParameters();
                    sslParameters.setEndpointIdentificationAlgorithm("HTTPS");
                    serverSocket.setSSLParameters(sslParameters);
                    n = false;
                }
            }
            serverSocket.startHandshake();
            this.session = serverSocket.getSession();
            this.serverSocket = serverSocket;
            try {
                this.serverOutput = new PrintStream(new BufferedOutputStream(this.serverSocket.getOutputStream()), false, HttpsClient.encoding);
            }
            catch (UnsupportedEncodingException ex3) {
                throw new InternalError(HttpsClient.encoding + " encoding not found");
            }
            if (!(n == 0)) {
                this.checkURLSpoofing(this.hv);
            }
        }
        else {
            this.session = ((SSLSocket)this.serverSocket).getSession();
        }
    }
    
    private void checkURLSpoofing(HostnameVerifier hostnameVerifier) throws IOException {
        String expectedName = this.url.getHost();
        if (expectedName != null && expectedName.startsWith("[") && expectedName.endsWith("]")) {
            expectedName = expectedName.substring(1, expectedName.length() - 1);
        }
        String cipherSuite = this.session.getCipherSuite();
        try {
            HostnameChecker instance = HostnameChecker.getInstance((byte)1);
            if (cipherSuite.startsWith("TLS_KRB5")) {
                if (!HostnameChecker.match(expectedName, this.getPeerPrincipal())) {
                    throw new SSLPeerUnverifiedException("Hostname checker failed for Kerberos");
                }
            }
            else {
                Certificate[] peerCertificates = this.session.getPeerCertificates();
                if ((peerCertificates[0] instanceof X509Certificate)) {
                    instance.match(expectedName, (X509Certificate)peerCertificates[0]);
                }
                else {
                    throw new SSLPeerUnverifiedException("");
                }
            }
            return;
        }
        catch (SSLPeerUnverifiedException ex) {}
        catch (CertificateException ex2) {}
        if (cipherSuite != null && cipherSuite.indexOf("_anon_") != -1) {
            return;
        }
        if (hostnameVerifier != null && hostnameVerifier.verify(expectedName, this.session)) {
            return;
        }
        this.serverSocket.close();
        this.session.invalidate();
        throw new IOException("HTTPS hostname wrong:  should be <" + this.url.getHost() + ">");
    }
    
    @Override
    protected void putInKeepAliveCache() {
        if (this.inCache) {
            assert false : "Duplicate put to keep alive cache";
            return;
        }
        else {
            this.inCache = true;
            HttpsClient.kac.put(this.url, this.sslSocketFactory, this);
            return;
        }
    }
    
    @Override
    public void closeIdleConnection() {
        HttpClient value = HttpsClient.kac.get(this.url, this.sslSocketFactory);
        if (value != null) {
            value.closeServer();
        }
    }
    
    String getCipherSuite() {
        return this.session.getCipherSuite();
    }
    
    public Certificate[] getLocalCertificates() {
        return this.session.getLocalCertificates();
    }
    
    Certificate[] getServerCertificates() throws SSLPeerUnverifiedException {
        return this.session.getPeerCertificates();
    }
    
    javax.security.cert.X509Certificate[] getServerCertificateChain() throws SSLPeerUnverifiedException {
        return this.session.getPeerCertificateChain();
    }
    
    Principal getPeerPrincipal() throws SSLPeerUnverifiedException {
        Principal principal;
        try {
            principal = this.session.getPeerPrincipal();
        }
        catch (AbstractMethodError abstractMethodError) {
            principal = ((X509Certificate)this.session.getPeerCertificates()[0]).getSubjectX500Principal();
        }
        return principal;
    }
    
    Principal getLocalPrincipal() {
        Principal principal;
        try {
            principal = this.session.getLocalPrincipal();
        }
        catch (AbstractMethodError abstractMethodError) {
            principal = null;
            Certificate[] localCertificates = this.session.getLocalCertificates();
            if (localCertificates != null) {
                principal = ((X509Certificate)localCertificates[0]).getSubjectX500Principal();
            }
        }
        return principal;
    }
    
    @Override
    public void handshakeCompleted(HandshakeCompletedEvent handshakeCompletedEvent) {
        this.session = handshakeCompletedEvent.getSession();
    }
    
    @Override
    public String getProxyHostUsed() {
        if (!this.needsTunneling()) {
            return null;
        }
        return super.getProxyHostUsed();
    }
    
    @Override
    public int getProxyPortUsed() {
        return (this.proxy == null || this.proxy.type() == Proxy.Type.DIRECT || this.proxy.type() == Proxy.Type.SOCKS) ? -1 : ((InetSocketAddress)this.proxy.address()).getPort();
    }
}
